@@ -3,13 +3,12 @@ use Mojo::Base -base;
 
 use Carp ();
 use JSON::Validator::OpenAPI::Mojolicious;
-use Mojo::JSON 'encode_json';
 use Mojo::UserAgent;
 use Mojo::Util;
 
 use constant DEBUG => $ENV{MOJO_OPENAPI_DEBUG} || 0;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 my $BASE = __PACKAGE__;
 
@@ -22,6 +21,16 @@ has base_url => sub {
     ->scheme($schemes->[0] || 'http');
 };
 
+has pre_processor => sub {
+  return sub {
+    my ($headers, $req) = @_;
+    return $headers, json => delete $req->{body} if ref $req->{body};
+    return $headers, form => $req->{form}        if defined $req->{form};
+    return $headers, $req->{body} if defined $req->{body};
+    return $headers;
+  };
+};
+
 has ua => sub { Mojo::UserAgent->new };
 
 sub new {
@@ -133,7 +142,7 @@ sub _generate_tx {
       $req{form}{$name} = $params->{$name};
     }
     elsif ($in eq 'body') {
-      @body = (ref $params->{$name} ? encode_json $params->{$name} : $params->{$name});
+      $req{body} = $params->{$name};
     }
     else {
       warn "[OpenAPI] Unknown 'in' '$in' for parameter '$name'";
@@ -142,13 +151,13 @@ sub _generate_tx {
 
   # Valid input
   warn "[OpenAPI] Input validation for '$url': @{@errors ? \@errors : ['Success']}\n" if DEBUG;
-  return $self->ua->build_tx($http_method, $url, \%headers, %req, @body) unless @errors;
+  return $self->ua->build_tx($http_method, $url, $self->pre_processor->(\%headers, \%req)) unless @errors;
 
   # Invalid input
   my $tx = Mojo::Transaction::HTTP->new;
   $tx->req->url($url);
   $tx->res->headers->content_type('application/json');
-  $tx->res->body(encode_json {errors => \@errors});
+  $tx->res->body(Mojo::JSON::encode_json({errors => \@errors}));
   $tx->res->code(400)->message($tx->res->default_message);
   $tx->res->error({message => 'Invalid input', code => 400});
   return $tx;
@@ -181,8 +190,8 @@ specification, with methods that transform parameters into a HTTP request.
 The generated class will perform input validation, so invalid data won't be
 sent to the server.
 
-Not that this implementation is currently EXPERIMENTAL! Feedback is
-appreciated.
+Not that this implementation is currently EXPERIMENTAL, but unlikely to change!
+Feedback is appreciated.
 
 =head1 SYNOPSIS
 
@@ -245,6 +254,28 @@ the Open API document:
 Returns a L<Mojo::URL> object with the base URL to the API. The default value
 comes from C<schemes>, C<basePath> and C<host> in the Open API specification.
 
+=head2 pre_processor
+
+  $code = $self->pre_processor;
+  $self = $self->pre_processor(sub { my ($headers, $req) = @_; ... });
+
+Holds a code ref that can pre-process the request. The return values are passed
+on to L<Mojo::UserAgent/build_tx>. Example:
+
+  $self->pre_processor(sub {
+    my ($headers, $req) = @_;
+    return $headers, json => {whatever => 42};
+  });
+
+The code above will result in this:
+
+  $self->ua->build_tx($http_method, $url, $headers, json => {whatever => 42});
+                                          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+
+C<$headers> is a hash-ref containing the request headers and C<$req> is a
+hash-ref that can contain either the key "body" or "form". Note that additional
+parameters might be added to C<$req>, though it is unlikely.
+
 =head2 ua
 
   $ua = $self->ua;
